/*
 * RotaryEncoder.cpp
 *
 *  Created on: 30 de jan. de 2022
 *      Author: igor_
 */

#include "RotaryEncoder.hpp"

#include "esp_timer.h"
#include "driver/gpio.h"

#define ESP_INTR_FLAG_DEFAULT 0

static void outside_class_isr_handler(void* arg) {
	enc_cnt++;
}

namespace RotaryEncoder{

void encoder::inside_class_isr_handler(void* arg) {
	enc_cnt++;
}

void encoder::encoder_init(encoder_config_t *enc_conf) {

	printf("Printing Encoder Configs:\n"
			"\tEnc1: %d\n"
			"\tEnc2: %d\n"
			"\tPtr: %p\n", enc_conf->enc1_pin, enc_conf->enc2_pin, this);

	this->enc1_pin = enc_conf->enc1_pin;
	this->enc2_pin = enc_conf->enc2_pin;

	// GPIO configuration structure
	gpio_config_t gpio_conf;

	gpio_conf.intr_type = GPIO_INTR_ANYEDGE;
	gpio_conf.mode = GPIO_MODE_INPUT;
	gpio_conf.pin_bit_mask = 1ULL<<this->enc1_pin | 1ULL<<this->enc2_pin;
	gpio_conf.pull_down_en = GPIO_PULLDOWN_DISABLE;
	gpio_conf.pull_up_en = GPIO_PULLUP_ENABLE;

	// Configures the GPIOs
	gpio_config(&gpio_conf);

	//install gpio isr service
	printf("----Install ISR Service %x----\n",
			gpio_install_isr_service(ESP_INTR_FLAG_DEFAULT));

	encoder_isr_config_t isr_args;
	isr_args.encoder_ptr = this;

	//hook isr handler for specific gpio pin
	if (enc_conf->interrupt == 1) {

		isr_args.gpio_num = enc_conf->enc1_pin;
		printf("----Install Inside ISR Handler 1: %x----\n",
				gpio_isr_handler_add(this->enc1_pin, this->inside_class_isr_handler, (void*) &isr_args));

		isr_args.gpio_num = enc_conf->enc2_pin;
		printf("----Install Inside ISR Handler 2: %x----\n",
				gpio_isr_handler_add(this->enc2_pin, this->inside_class_isr_handler, (void*) &isr_args));
	}
	else if (enc_conf->interrupt == 2){

		isr_args.gpio_num = enc_conf->enc1_pin;
		printf("----Install Outside ISR Handler 1: %x----\n",
				gpio_isr_handler_add(this->enc1_pin, outside_class_isr_handler, (void*) &isr_args));

		isr_args.gpio_num = enc_conf->enc2_pin;
		printf("----Install Outside ISR Handler 2: %x----\n",
				gpio_isr_handler_add(this->enc2_pin, outside_class_isr_handler, (void*) &isr_args));
	}
}
}





