#include "esp_event.h"
#include "driver/gpio.h"

#include "esp_timer.h"
#include "RotaryEncoder.hpp"

#define ESP_INTR_FLAG_DEFAULT 0

uint32_t cnt;

static void IRAM_ATTR main_isr_handler(void* arg)
{
	cnt++;
}

extern "C" void app_main(void)
{
	encoder_t encoder;

	encoder_config_t conf;

	conf.enc1_pin = GPIO_NUM_34;
	conf.enc2_pin = GPIO_NUM_35;

	/* Interrupt = 0 -> Interrupt in Main
	 * Interrupt = 1 -> Interrupt Inside Class
	 * Interrupt = 2 -> Interrupt Outside Class
	 */
	conf.interrupt = 0;

	if(conf.interrupt == 0) {

		gpio_config_t gpio_conf;

		gpio_conf.intr_type = GPIO_INTR_ANYEDGE;
		gpio_conf.mode = GPIO_MODE_INPUT;
		gpio_conf.pin_bit_mask = 1ULL<<conf.enc1_pin | 1ULL<<conf.enc2_pin;
		gpio_conf.pull_down_en = GPIO_PULLDOWN_DISABLE;
		gpio_conf.pull_up_en = GPIO_PULLUP_ENABLE;

		gpio_config(&gpio_conf);

		printf("----Install ISR Service %x----\n",
				gpio_install_isr_service(ESP_INTR_FLAG_DEFAULT));

		printf("----Install Main ISR Handler 1: %x----\n",
			gpio_isr_handler_add(GPIO_NUM_34, main_isr_handler, (void*) 34));
		printf("----Install Main ISR Handler 1: %x----\n",
			gpio_isr_handler_add(GPIO_NUM_35, main_isr_handler, (void*) 35));
	}
	else {

		encoder.encoder_init(&conf);
	}

    gpio_set_direction(GPIO_NUM_32, GPIO_MODE_OUTPUT);
    gpio_set_direction(GPIO_NUM_33, GPIO_MODE_OUTPUT);
	int level = 0;

    while (true) {
        gpio_set_level(GPIO_NUM_32, level);
        gpio_set_level(GPIO_NUM_33, !level);
        level = !level;
    	printf("Counter Main: %d\t\tCounter Class: %d\n", cnt, enc_cnt);
        vTaskDelay(500 / portTICK_PERIOD_MS);
    }
}
